# -*- coding: utf-8 -*-
from pkg_resources import DistributionNotFound
from pkg_resources import get_distribution

from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone import api
from pybars import Compiler
from zope.i18n import translate

import os.path
import sys

try:
    get_distribution('plone.tiles')
except DistributionNotFound:   # pragma: no cover
    class Tile(BrowserView):
        """Fake Tile which is only a BrowserView"""
    class PersistentTile(BrowserView):
        """Fake persistent Tile which is only a BrowserView"""
else:    # pragma: no cover
    from plone.tiles.tile import Tile
    from plone.tiles.tile import PersistentTile


# global handlebars compiler
compiler = Compiler()
HBS_REGISTRY = {}


def package_home(gdict):
    filename = gdict["__file__"]
    return os.path.dirname(filename)


class HandlebarsMixin:

    def get_contents(self):
        """ Get CMS data and put it in a JSON format for hbs inclusion

        :return: dictonary (must not be a JSON structure!
                 The conversion is handled by the templating engine)
        """
        raise NotImplementedError

    def _get_partial_key(self, partial_filename):
        """
        :param partial_filename: full path of partial
               (/home/vagrant/templates/_slideshow.js.hbs)
        :return: key for partial inclusion (_slideshow.js)
        """
        path, filename = os.path.split(partial_filename)
        partial_name, ext = os.path.splitext(filename)
        return partial_name

    def _get_hbs_template(self, hbs_filename):
        """
        :param hbs_filename: full path of partial
               (/home/vagrant/templates/_slideshow.js.hbs)
        :return: compiled hbs template
        """
        if hbs_filename in HBS_REGISTRY:
            compiled_template = HBS_REGISTRY[hbs_filename]
        else:
            with open(hbs_filename) as f:
                hbs_template = unicode(f.read(), 'utf-8')
                compiled_template = compiler.compile(hbs_template)
                HBS_REGISTRY[hbs_filename] = compiled_template
        return compiled_template

    def get_partials(self, hbs_dir):
        """ Get partials for rendering the master hbs_template

        :param hbs_dir: directory of master template
               (/home/vagrant/templates/)
        :return: dictonary with compiled partials templates
                 ({'subitem': <template 'subitem'>})
        """
        return {}

    def get_helpers(self):
        """ Get helpers for rendering the master hbs_template

        :return: dictonary with compiled partials templates
                 ({'list': self.list_items()})
        """
        return {}

    def hbs_snippet(self, filename=None, _prefix=None):
        if filename:
            # first scenario: get snippet from filename
            path = self.get_path_from_prefix(_prefix)
            hbs_file = os.path.join(path, filename)
            if not os.path.isfile(hbs_file):
                raise ValueError('No such file', hbs_file)
        elif hasattr(self, 'index'):  # noqa
            # second scenario: get snippet from zcml
            # reuse filename from tile definition in zcml but read file here
            # otherwise it is interpreted as XML/PT
            hbs_file = self.index.filename
        else:
            raise ValueError('No template found!')

        hbs_template = self._get_hbs_template(hbs_file)
        hbs_dir = os.path.dirname(hbs_file)
        partials = self.get_partials(hbs_dir)
        helpers = self.get_helpers()

        return hbs_template(self.get_contents(),
                            helpers=helpers,
                            partials=partials)

    def get_path_from_prefix(self, _prefix):
        if isinstance(_prefix, str):
            path = _prefix
        else:
            if _prefix is None:
                _prefix = sys._getframe(2).f_globals
            path = package_home(_prefix)
        return path

    def translate(self, msgid, domain=None, mapping=None, context=None,
                  target_language=None, default=None):
        if not target_language:
            target_language = api.portal.get_current_language(context)
        return translate(msgid=msgid,
                         domain=domain,
                         mapping=mapping,
                         context=context,
                         default=default,
                         target_language=target_language)


class HandlebarsBrowserView(BrowserView, HandlebarsMixin):
    """ A simple browserview using hbs as templating engine"""

    def __call__(self, *args, **kwargs):
        return self.hbs_snippet()


class HandlebarsPloneView(BrowserView, HandlebarsMixin):
    """ A hbs view rendered in the content slot of the main template of Plone
    """

    main_template = ViewPageTemplateFile('templates/plone_standard_template.pt')  # noqa

    def __call__(self, *args, **kwargs):
        return self.main_template(*args, **kwargs)


class HandlebarsTile(Tile, HandlebarsMixin):

    def __call__(self, *args, **kwargs):
        return self.hbs_snippet()


# BBB
HandlebarTile = HandlebarsTile


class HandlebarsPersistentTile(PersistentTile, HandlebarsMixin):

    def __call__(self, *args, **kwargs):
        return self.hbs_snippet()

# EOF



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from collective.handlebars.testing import COLLECTIVE_HANDLEBARS_INTEGRATION_TESTING  # noqa
from collective.handlebars.browser.views import HandlebarsBrowserView
from collective.handlebars.browser.views import HandlebarsTile
from collective.handlebars.browser.views import HandlebarsPersistentTile

from plone import api
from zope.i18nmessageid import MessageFactory
from zope import component, interface
from zope.i18n.interfaces import ITranslationDomain
import os.path

import unittest


TEST_DATA__DIR = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                              'data')
_ = MessageFactory('my.domain')


class DummyHbsTile(HandlebarsTile):

    def get_contents(self):
        return {'src': 'foo', 'alt': 'bar'}


class DummyHbsPersistentTile(HandlebarsPersistentTile):

    def get_contents(self):
        return {'src': 'foo', 'alt': 'bar'}


class DummyHbsTemplate(object):

    def __init__(self, filename):
        self.filename = filename


@interface.implementer(ITranslationDomain)
class TestDomain(dict):

    def translate(self, text, *_, **__):
        return self[text], _[2]


class DummyHbsFile(HandlebarsBrowserView):

    def get_contents(self):
        return {'title': u'File Test',
                'body': u'Hello HBS World!'}

    def __call__(self):
        return self.hbs_snippet('data/minimal.hbs')

    def invalid(self):
        return self.hbs_snippet('data/minimal_notfound.hbs')


class HelperHbsView(HandlebarsBrowserView):

    def get_contents(self):
        return {'title': u'File Test',
                'body': u'Hello HBS Wörld!'}

    def _myhelper(self, options, items):
        return u'HELPER: ' + items

    def get_helpers(self):
        return {'myhelper': self._myhelper}

    def __call__(self):
        return self.hbs_snippet('data/helper.hbs')


class TestBrowserView(unittest.TestCase):
    """Test the collective.handlebars BrowserView component."""

    layer = COLLECTIVE_HANDLEBARS_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.view = self.portal.restrictedTraverse('@@hbs_test_view')

    def test_get_contents_default(self):
        """ Method `get_tile_data` is not implemented in base class

        :return:
        """
        view = HandlebarsBrowserView(self.portal, self.layer['request'])
        self.assertRaises(NotImplementedError, view.get_contents)

    def test_base_view(self):
        """ An error is raised, if no template is specified """
        view = HandlebarsBrowserView(self.portal, self.layer['request'])
        self.assertRaises(ValueError, view)

    def test_example_view(self):
        """Test that a handlebars template is rendered."""
        view = self.portal.restrictedTraverse('@@hbs_test_view')
        result_file = open(os.path.join(TEST_DATA__DIR, 'minimal.html'))
        self.assertEqual(view(), unicode(result_file.read(), encoding='utf-8'))

    def test_translate(self):
        view = HandlebarsBrowserView(self.portal, self.layer['request'])
        component.provideUtility(TestDomain(Allowed=_('Erlaubt')),
                                 name='my.domain')
        self.assertEqual(view.translate(_('Allowed'), target_language='de'),
                         (u'Erlaubt', 'de'))

    def test_translate_default_lang(self):
        view = HandlebarsBrowserView(self.portal, self.layer['request'])
        component.provideUtility(TestDomain(Allowed=_('Allowed')),
                                 name='my.domain')
        self.assertEqual(view.translate(_('Allowed')), (u'Allowed', 'en'))

    def test_hbs_snippet(self):
        view = DummyHbsFile(self.portal, self.layer['request'])
        result_file = open(os.path.join(TEST_DATA__DIR, 'minimal_file.html'))
        self.assertEqual(view(), unicode(result_file.read(), encoding='utf-8'))

    def test_hbs_snippet_nofile(self):
        view = DummyHbsFile(self.portal, self.layer['request'])
        self.assertRaises(ValueError, view.invalid)

    def test_get_path_from_prefix(self):
        view = DummyHbsFile(self.portal, self.layer['request'])
        self.assertEqual(view.get_path_from_prefix(_prefix='.'), '.')

    def test_helpers(self):
        view = HelperHbsView(self.portal, self.layer['request'])
        result_file = open(os.path.join(TEST_DATA__DIR, 'helper.html'))
        self.assertEqual(view(), unicode(result_file.read(), encoding='utf-8'))


class TestPloneView(unittest.TestCase):
    """Test the collective.handlebars BrowserView component."""

    layer = COLLECTIVE_HANDLEBARS_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.view = self.portal.restrictedTraverse('@@hbs_test_ploneview')

    def test_get_contents_default(self):
        """ Method `get_tile_data` is not implemented in base class

        :return:
        """
        view = HandlebarsBrowserView(self.portal, self.layer['request'])
        self.assertRaises(NotImplementedError, view.get_contents)

    def test_base_view(self):
        """ An error is raised, if no template is specified """
        view = HandlebarsBrowserView(self.portal, self.layer['request'])
        self.assertRaises(ValueError, view)

    def test_example_view(self):
        """Test that a handlebars template is rendered."""
        view = self.portal.restrictedTraverse('@@hbs_test_ploneview')
        result_file = open(os.path.join(TEST_DATA__DIR, 'minimal_plone.html'))
        self.assertIn(unicode(result_file.read(), encoding='utf-8'), view())


class TestUninstall(unittest.TestCase):

    layer = COLLECTIVE_HANDLEBARS_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')
        self.installer.uninstallProducts(['collective.handlebars'])

    def test_product_uninstalled(self):
        """Test if collective.handlebars is cleanly uninstalled."""
        self.assertFalse(self.installer.isProductInstalled(
            'collective.handlebars'))

    def test_browserlayer_removed(self):
        """Test that ICollectiveHandlebarsLayer is removed."""
        from collective.handlebars.interfaces import ICollectiveHandlebarsLayer
        from plone.browserlayer import utils
        self.assertNotIn(ICollectiveHandlebarsLayer, utils.registered_layers())


class TestHandlebarTile(unittest.TestCase):
    """Test that fhnw.web16theme viewlets."""

    layer = COLLECTIVE_HANDLEBARS_INTEGRATION_TESTING

    def setUp(self):
        self.tile = HandlebarsTile(self.layer['portal'], self.layer['request'])
        self.template_path = os.path.join(TEST_DATA__DIR,
                                          '_slideshow_slide.js.hbs')

    def test_get_contents_default(self):
        """ Method `get_tile_data` is not implemented in base class

        :return:
        """
        view = HandlebarsBrowserView(self.layer['portal'],
                                     self.layer['request'])
        self.assertRaises(NotImplementedError, view.get_contents)

    def test_get_hbs_template(self):
        template = self.tile._get_hbs_template(self.template_path)
        self.assertEqual(template({'src': 'foo', 'alt': 'bar'}).strip(),
                         u'<img src="foo" alt="bar">')

    def test_get_partial_key(self):
        self.assertEqual(self.tile._get_partial_key(self.template_path),
                         '_slideshow_slide.js')

    def test_call_notemplate(self):
        """ An error is raised, if no template is specified """
        self.assertRaises(ValueError, self.tile)

    def test_call_with_template(self):
        tile = DummyHbsTile(self.layer['portal'], self.layer['request'])
        setattr(tile, 'index', DummyHbsTemplate(self.template_path))
        self.assertEqual(tile().strip(), u'<img src="foo" alt="bar">')


class TestHandlebarPersistentTile(unittest.TestCase):
    """Test that fhnw.web16theme viewlets."""

    layer = COLLECTIVE_HANDLEBARS_INTEGRATION_TESTING

    def setUp(self):
        self.tile = HandlebarsPersistentTile(
            self.layer['portal'], self.layer['request'])
        self.template_path = os.path.join(TEST_DATA__DIR,
                                          '_slideshow_slide.js.hbs')

    def test_get_contents_default(self):
        """ Method `get_tile_data` is not implemented in base class

        :return:
        """
        view = HandlebarsBrowserView(self.layer['portal'],
                                     self.layer['request'])
        self.assertRaises(NotImplementedError, view.get_contents)

    def test_get_hbs_template(self):
        template = self.tile._get_hbs_template(self.template_path)
        self.assertEqual(template({'src': 'foo', 'alt': 'bar'}).strip(),
                         u'<img src="foo" alt="bar">')

    def test_get_partial_key(self):
        self.assertEqual(self.tile._get_partial_key(self.template_path),
                         '_slideshow_slide.js')

    def test_call_notemplate(self):
        """ An error is raised, if no template is specified """
        self.assertRaises(ValueError, self.tile)

    def test_call_with_template(self):
        tile = DummyHbsTile(self.layer['portal'], self.layer['request'])
        setattr(tile, 'index', DummyHbsTemplate(self.template_path))
        self.assertEqual(tile().strip(), u'<img src="foo" alt="bar">')



# -*- coding: utf-8 -*-
from collective.handlebars.testing import COLLECTIVE_HANDLEBARS_ACCEPTANCE_TESTING  # noqa
from plone.app.testing import ROBOT_TEST_LEVEL
from plone.testing import layered

import os
import robotsuite
import unittest


def test_suite():
    suite = unittest.TestSuite()
    current_dir = os.path.abspath(os.path.dirname(__file__))
    robot_dir = os.path.join(current_dir, 'robot')
    robot_tests = [
        os.path.join('robot', doc) for doc in os.listdir(robot_dir)
        if doc.endswith('.robot') and doc.startswith('test_')
    ]
    for robot_test in robot_tests:
        robottestsuite = robotsuite.RobotTestSuite(robot_test)
        robottestsuite.level = ROBOT_TEST_LEVEL
        suite.addTests([
            layered(
                robottestsuite,
                layer=COLLECTIVE_HANDLEBARS_ACCEPTANCE_TESTING
            ),
        ])
    return suite



# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from collective.handlebars.testing import COLLECTIVE_HANDLEBARS_INTEGRATION_TESTING  # noqa
from plone import api

import unittest


class TestSetup(unittest.TestCase):
    """Test that collective.handlebars is properly installed."""

    layer = COLLECTIVE_HANDLEBARS_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_product_installed(self):
        """Test if collective.handlebars is installed."""
        self.assertTrue(self.installer.isProductInstalled(
            'collective.handlebars'))

    def test_browserlayer(self):
        """Test that ICollectiveHandlebarsLayer is registered."""
        from collective.handlebars.interfaces import (
            ICollectiveHandlebarsLayer)
        from plone.browserlayer import utils
        self.assertIn(ICollectiveHandlebarsLayer, utils.registered_layers())


class TestUninstall(unittest.TestCase):

    layer = COLLECTIVE_HANDLEBARS_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')
        self.installer.uninstallProducts(['collective.handlebars'])

    def test_product_uninstalled(self):
        """Test if collective.handlebars is cleanly uninstalled."""
        self.assertFalse(self.installer.isProductInstalled(
            'collective.handlebars'))

    def test_browserlayer_removed(self):
        """Test that ICollectiveHandlebarsLayer is removed."""
        from collective.handlebars.interfaces import ICollectiveHandlebarsLayer
        from plone.browserlayer import utils
        self.assertNotIn(ICollectiveHandlebarsLayer, utils.registered_layers())

    def test_hiddenprofiles(self):
        """ Test uninstall profile is hidden
        :return:None
        """
        from collective.handlebars.setuphandlers import HiddenProfiles
        self.assertIn('collective.handlebars:default',
                      HiddenProfiles().getNonInstallableProfiles())
        self.assertIn('collective.handlebars:uninstall',
                      HiddenProfiles().getNonInstallableProfiles())
